import numpy as np
import cv2
import skvideo.io
import pyaudio
import time
from multiprocessing.pool import ThreadPool

# function to read and store audio on thread

def audio(extract):

    # read samples for frame from stream to buffer

    data = stream.read(chunk)

    # convert to float

    data = np.frombuffer(data, dtype=np.float32)

    audio_list.append(data)

    return(data)


if __name__ == "__main__":
      
  # set variables

  # video

  aspect_ratio = 4/3 # set the aspect ratio
  fps = 60 # set the frames per second - This needs to be the same as the camera frame rate
  input_camera = 1 # set the input camera device, 0 = internal camera, 1 = external camera
  aspect_x = 64 # aspect ratio for x
  aspect_y = 48 #  aspect ratio for y - both x and y are temps, will default to nearest and these are taken later in script

  # audio

  input_audio_device = 0 # sets inut device to read audio
                         # On Windows this should be the stereo mix
                         # to find correct device index run the device index script
  sample_format = pyaudio.paFloat32 # set to 32-bit float
  channels = 1 # set to mono audio
  fs = 8000 # set the sampling rate

  while fs%fps != 0: # ensure that the number of audio samples per frame if an integer
        fs += 1

  print('The closest sampling rate divisible by ' + str(fps) + ' fps to return an integer is: ' + str(fs)) # This is the fs
                                                                                                           # that should be
                                                                                                           # use in future
                                                                                                           # scripts

  chunk = int(fs/fps) # set the number of audio samples to index for each frame of video
      
  # initialise video

  vid = cv2.VideoCapture(input_camera)

  # set fps

  vid.set(cv2.CAP_PROP_FPS, fps)
  
  # set initial resolution (will default to nearest)

  vid.set(cv2.CAP_PROP_FRAME_WIDTH, 1280)
  vid.set(cv2.CAP_PROP_FRAME_HEIGHT, 720)

  # get actual resolution

  width = vid.get(3)
  height = vid.get(4)

  # set new width

  new_width = height*aspect_ratio

  # find difference in widths

  difference = width - new_width

  # find array values to trim

  left = difference/2
  right = width - left

  # declare list to append frames

  video_list = []

  # list to store frames

  audio_list = []

  # create instance of pyaudio

  p = pyaudio.PyAudio()

  # create stream object

  stream = p.open(format=sample_format,
                  channels=channels,
                  rate=fs,
                  frames_per_buffer=chunk,
                  input_device_index = input_audio_device,
                  input=True)

  # declare framecount

  framecount = 0

  # declare thread pool

  pool = ThreadPool(processes=1)

  # time for start of recording

  recstart = time.time()


  while(True):

      # time for start of frame

      framestart = time.time()

      async_result = pool.apply_async(audio, (chunk,)) 

      # video

      # read the frame from camera

      ret, frame = vid.read()

      # trim the frame to 4:3

      frame = frame[0:, int(left):int(right)]

      # lower resolution

      frame = cv2.resize(frame, (aspect_x, aspect_y))

      # flip the image

      frame = cv2.flip(frame, 1)

      # convert to greyscale

      frame = cv2.cvtColor(frame, cv2.COLOR_BGR2GRAY)

      # create resizable window for display

      cv2.namedWindow('frame', cv2.WINDOW_NORMAL)

      # resize the window

      cv2.resizeWindow('frame', int(new_width), int(height))

      # display frame

      cv2.imshow('frame', frame)

      # append frame to list

      video_list.append(frame)

      # time for end of frame

      frameend = time.time()

      print('frame: ' + str(framecount) + ', length: ' + str(frameend-framestart))

      framecount += 1

      # break if q key pressed
          
      if cv2.waitKey(1) & 0xFF == ord('q'):
          break

  # Stop and close the stream 

  stream.stop_stream()
  stream.close()

  # Terminate the PortAudio interface

  p.terminate()

  recend = time.time()

  print('recording length: ' + str(recend-recstart))

  # convert list to array

  video_array = np.array(video_list)

  # save video as numpy array

  np.save('./Temp/video_capture.npy', video_array)

  # export video as mp4

  vid_out = skvideo.io.FFmpegWriter('./Temp/video.mp4', inputdict={
        '-r': '60',
      },
      outputdict={
        '-vcodec': 'libx264',
        '-pix_fmt': 'yuv444p',
        '-r': '60',
        '-crf': '0',
  })

  for frame in video_array:
      vid_out.writeFrame(frame)

  vid_out.close()

  # save audio numpy array

  audio_array = np.array(audio_list)

  np.save('./Temp/audio_capture.npy', audio_array)


