import numpy as np
import time
from Utils import motion
from Utils import feat_examples_size


if __name__ == "__main__":

    # set variables

    fs = 8040 # set sampling frequency - should be set the same as was returned in capture script
    fps = 60 # set camera frames per second
    chunk = int(fs/fps) # find the number of audio samples per frame
    segment_length = 0.5 # length of sample segment in seconds
    segment_frames = int(fps*segment_length) # set the number of video frames per segment
    segment_chunk = int(chunk*segment_frames) # set the number of audio samples per segment

    # find number of examples and features

    no_of_feats_x, no_of_feats_y, no_of_examples = feat_examples_size(segment_length)

    # set paths where video and audio files are

    audios = np.load('./Temp/audio_capture.npy')
    videos = np.load('./Temp/video_capture.npy')

    # declare empty arrays for features and targets

    motion_features_x = np.zeros((no_of_examples, no_of_feats_x[0], no_of_feats_x[1], 1))
    motion_features_y = np.zeros((no_of_examples, no_of_feats_y[0], no_of_feats_y[1], 1))
    audio_features = np.zeros((no_of_examples, segment_chunk))

    start_time = time.time()

    # declare initial start and end frames for slicing

    start = 0
    end = segment_frames

    for i in range(len(videos)-segment_frames):

        print('segment: ' + str(start))

        if end > len(videos):

            continue

        else:

            # slice audio and video

            video_segment = videos[start:end]
            audio_segment = audios[start:end]

            # extract x and y motiongrams and reshape for training

            mg_x, mg_y, _, _, _ = motion(video_segment)
            mg_x = mg_x.reshape(mg_x.shape[0], mg_x.shape[1], 1)
            mg_y = mg_y.reshape(mg_y.shape[0], mg_y.shape[1], 1)

            # flatten audio segment

            target = audio_segment.flatten()

            # place in final arrays

            motion_features_x[start,:] = mg_x
            motion_features_y[start,:] = mg_y
            audio_features[start,:] = target
                
            start += 1
            end += 1
                
    end_time = time.time()
    print(end_time-start_time)

    np.save('./Temp/motion_features_x.npy', motion_features_x)
    np.save('./Temp/motion_features_y.npy', motion_features_y)
    np.save('./Temp/audio_features.npy', audio_features)