import numpy as np
import scipy


def lin_interp_1d(data, out_size):
    
    in_size = data.shape[0]
    x_in = np.arange(0,in_size)
    interpolator = scipy.interpolate.interp1d(x_in, data)
    x_out = np.arange(0,in_size-1,((in_size-1)/out_size))
    output = interpolator(x_out)
    output = output[0:out_size]
    
    return output

def motion(clip):
    
    
    ### Centroid of Motion taken from Musical Gestures Toolbox and reworked here
    
    
    ### declaring empty arrays ###
    
    # declare empty arrays for mgs
    
    mg_x = np.zeros((clip.shape[0], clip.shape[2]), dtype=np.uint8)
    mg_y = np.zeros((clip.shape[1], clip.shape[0]), dtype=np.uint8)
    
    # declare empty array for QOM
    
    qom = np.zeros(clip.shape[0])
    
    # declare empty arrays for COMs
    
    com_x = np.zeros(clip.shape[0])
    com_y = np.zeros(clip.shape[0])
    
    ### calculating frame difference ###
    
    # declare empty arrays for frame difference 
    
    frame_diff = np.zeros((clip.shape[0]-1, clip.shape[1], clip.shape[2]), dtype=np.float16)
    
    # calculate absolute frame difference
    
    for i in range(1, len(clip)):
        frame_diff[i-1] = np.abs(clip[i].astype(float) - clip[i-1].astype(float))        
        
        
    ### per frame operations ###
        
    # calculate mean of each frame difference for x and y
        
    for i in range(len(frame_diff)):
        
        mean_x = np.mean(frame_diff[i], axis=0)
        mean_y = np.mean(frame_diff[i], axis=1)
        
        ### Motiongrams ###
        
        mg_x[i] = mean_x
        mg_y[:,i] = mean_y
        
        ### QOM ###
        
        # calculate number of pixels changed
    
        pixel_change = np.where(frame_diff[i] != 0)
        qom[i] = pixel_change[0].size
        
        ### COM ### - this taken and reworked from Musical Gestures Toolbox
        
        mean_x_im = np.mean(clip[i], axis=0)
        mean_y_im = np.mean(clip[i], axis=1)
        
        x = np.arange(clip.shape[2])
        y = np.arange(clip.shape[1])
        
        if np.sum(mean_x_im) != 0 and np.sum(mean_y_im) != 0:
            com_x[i] = x.reshape(1, clip.shape[2])@mean_x_im.reshape(clip.shape[2], 1)/np.sum(mean_x_im)
            com_y[i] = clip.shape[1] - (y.reshape(1, clip.shape[1])@mean_y_im.reshape(clip.shape[1], 1)/np.sum(mean_y_im))
        else:
            com_x[i] = 0
            com_y[i] = 0
        

            

            
        
    ### Operations on whole array ###
    
    ### Motiongrams ###
        
    # reduce range in mg to allow lesser motions to become a little clearer
    
    mg_x = np.clip(mg_x, 1, 7)
    mg_x = (mg_x/mg_x.max())*255
    mg_x = np.where(mg_x == mg_x.min(), 0, mg_x)
    
    mg_y = np.clip(mg_y, 1, 7)
    mg_y = (mg_y/mg_y.max())*255
    mg_y = np.where(mg_y == mg_y.min(), 0, mg_y)
    
    # convert to uint8
    
    mg_x = np.array(mg_x, dtype = np.uint8)
    mg_y = np.array(mg_y, dtype = np.uint8)
    
    ### QOM ###
        
    # normalise to total number of pixels
    
    qom = qom/(clip.shape[1]*clip.shape[2])
    
    ### COM ###
    
    # normalise within frame
    
    com_x = com_x/clip.shape[2]
    com_y = com_y/clip.shape[1]
    
    
    return mg_x, mg_y, qom, com_x, com_y

def feat_examples_size(sample_length):
    
    # load audio

    audios = np.load('./Temp/audio_capture.npy')

    # find size of grain

    fps = 60
    grain_time = sample_length
    grain_size = int(fps*grain_time)

    # declare examples counter

    no_of_examples = 0

    # count examples
            
    no_of_examples = len(audios) - (grain_size - 1)

    # find number of features

    # load video

    videos = np.load('./Temp/video_capture.npy')

    # create single example

    clip = videos[0:grain_size]

    # find total features

    mg_x, mg_y, _, _, _ = motion(clip)

    # mg_x = mg_x.flatten()
    # mg_y = mg_y.T.flatten()

    no_of_feats_x = mg_x.shape
    no_of_feats_y = mg_y.shape

    print('No of Example: ' + str(no_of_examples))
    print('No of Features_x: ' + str(no_of_feats_x))
    print('No of Features_y: ' + str(no_of_feats_y))

    return no_of_feats_x, no_of_feats_y, no_of_examples



    


        